@@ -376,7 +376,7 @@ aris_proof_destroy (aris_proof * ap)
 	  sen_data_destroy (y_sd);
 	}
 
-      ls_clear (ap->yanked);
+      destroy_list (ap->yanked);
       ap->yanked = NULL;
     }
 
@@ -402,7 +402,10 @@ aris_proof_destroy (aris_proof * ap)
 
   goal_destroy (ap->goal);
   if (ap->selected)
-    ls_clear (ap->selected);
+    {
+      destroy_list (ap->selected);
+      ap->selected = NULL;
+    }
 
   sen_parent_destroy ((sen_parent *) SEN_PARENT (ap));
 }
@@ -487,7 +490,7 @@ aris_proof_adjust_lines (aris_proof * ap, item_t * itm, int mod)
   int cur_line = ((sentence *) itm->value)->line_num;
   item_t * ev_itr;
 
-  for (ev_itr = itm->next; ev_itr != NULL; ev_itr = ev_itr->next)
+  for (ev_itr = itm->next; ev_itr; ev_itr = ev_itr->next)
     {
       int new_line_no, ret;
       sentence * ev_sen;
@@ -504,7 +507,7 @@ aris_proof_adjust_lines (aris_proof * ap, item_t * itm, int mod)
 	  // Check if any index is greater than or equal to
 	  // the new item's line number.
 	  if (ev_sen->indices[i] >= cur_line)
-	    ev_sen->indices[i]++;
+	    ev_sen->indices[i] = ev_sen->indices[i] + line_mod;
 	}
     }
 
@@ -529,7 +532,7 @@ aris_proof_to_proof (aris_proof * ap)
   if (!proof)
     return NULL;
 
-  for (g_itr = ap->goal->goals->head; g_itr != NULL; g_itr = g_itr->next)
+  for (g_itr = ap->goal->goals->head; g_itr; g_itr = g_itr->next)
     {
       sen = g_itr->value;
       unsigned char * entry_text = sen->text;
@@ -539,7 +542,7 @@ aris_proof_to_proof (aris_proof * ap)
 	return NULL;
     }
 
-  for (ev_itr = SEN_PARENT (ap)->everything->head; ev_itr != NULL; ev_itr = ev_itr->next)
+  for (ev_itr = SEN_PARENT (ap)->everything->head; ev_itr; ev_itr = ev_itr->next)
     {
       sen = ev_itr->value;
       sd = sentence_copy_to_data (sen);
@@ -593,7 +596,7 @@ aris_proof_create_sentence (aris_proof * ap, sen_data * sd)
     {
       sd->depth = ((sentence *) fcs->value)->depth - 1;
     }
-  else
+  else if (sd->depth == DEPTH_DEFAULT)
     {
       sd->depth = ((sentence *) fcs->value)->depth;
       if (sd->subproof)
@@ -633,7 +636,7 @@ aris_proof_create_new_prem (aris_proof * ap)
   sentence * sen;
   sen_data * sd;
 
-  sd = SEN_DATA_DEFAULT (1, 0, 0);
+  sd = SEN_DATA_DEFAULT (1, 0, DEPTH_DEFAULT);
   if (!sd)
     return NULL;
 
@@ -658,7 +661,7 @@ aris_proof_create_new_conc (aris_proof * ap)
   sentence * sen;
   sen_data * sd;
 
-  sd = SEN_DATA_DEFAULT (0, 0, 0);
+  sd = SEN_DATA_DEFAULT (0, 0, DEPTH_DEFAULT);
   if (!sd)
     return NULL;
 
@@ -683,7 +686,7 @@ aris_proof_create_new_sub (aris_proof * ap)
   sentence * sen;
   sen_data * sd;
 
-  sd = SEN_DATA_DEFAULT (0, 1, 0);
+  sd = SEN_DATA_DEFAULT (0, 1, DEPTH_DEFAULT);
   if (!sd)
     return NULL;
 
@@ -868,7 +871,7 @@ aris_proof_copy (aris_proof * ap)
       if (sel_itr->prev && sen->depth < ((sentence *) sel_itr->prev->value)->depth)
 	sd->depth = -1;
       else
-	sd->depth = 0;
+	sd->depth = DEPTH_DEFAULT;
 
       ret_chk = ls_ins_obj (ap->yanked, sd, ap->yanked->tail);
       if (!ret_chk)
@@ -886,7 +889,7 @@ aris_proof_copy (aris_proof * ap)
  *  input:
  *    ap - The aris proof from which sentences are being copied.
  *  output:
- *    0 on success, -1 on memory error.
+ *    0 on success, -1 on memory error, 1 if the sentence is the first.
  */
 int
 aris_proof_kill (aris_proof * ap)
@@ -896,7 +899,7 @@ aris_proof_kill (aris_proof * ap)
   if (ret_chk == -1)
     return -1;
 
-  item_t * sel_itr;
+  item_t * sel_itr, * start_itr;
   int is_subproof = 0;
   int org_depth = -1;
  
@@ -914,6 +917,17 @@ aris_proof_kill (aris_proof * ap)
       sel_itr = ap->selected->head;
     }
 
+  start_itr = sel_itr;
+
+  for (start_itr = sel_itr; start_itr; start_itr = start_itr->next)
+    {
+      sentence * sen;
+      sen = start_itr->value;
+
+      if (sen->line_num == 1)
+	return 1;
+    }
+
   for (; sel_itr; sel_itr = sel_itr->next)
     {
       sentence * sen;
@@ -1026,4 +1040,3 @@ aris_proof_toggle_boolean_mode (aris_proof * ap)
 
   return 0;
 }
-
