@@ -967,7 +967,7 @@ Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.");
 			 "comments", _("  GNU Aris is a logical proof program, which supports both propositional and predicate logic, as well as Boolean algebra and arithmetical logic in the form of abstract sequences."),
 			 "logo", the_app->icon,
 			 "wrap-license", TRUE,
-			 "website", _("<http://www.gnu.org/software/aris/"),
+			 "website", _("<http://www.gnu.org/software/aris/>"),
 			 NULL);
 
   return 0;
@@ -1097,7 +1097,10 @@ menu_activated (aris_proof * ap, int menu_id)
       if (ret < 0)
 	return -1;
 
-      aris_proof_set_sb (ap, _("Sentence Killed."));
+      if (ret == 1)
+	aris_proof_set_sb (ap, _("The first sentence can not be killed."));
+      else
+	aris_proof_set_sb (ap, _("Sentence Killed."));
       break;
     case MENU_INSERT:
       ret = aris_proof_yank (ap);
