@@ -24,6 +24,7 @@
 #include "rules.h"
 
 #include <ctype.h>
+#include <math.h>
 
 /* Eliminates a negation from a string.
  *  input:
@@ -38,11 +39,7 @@ elim_not (const unsigned char * not_str)
   unsigned int not_len = strlen ((const char *) not_str);
 
   ret = (unsigned char *) calloc (not_len, sizeof (char));
-  if (!ret)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (ret, NULL);
   strncpy (ret, not_str + NL, not_len - NL);
   ret[not_len - NL] = '\0';
 
@@ -68,11 +65,7 @@ elim_par (const unsigned char * par_str)
     return NULL;
 
   ret = (unsigned char *) calloc (par_len, sizeof (char));
-  if (!ret)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (ret, NULL);
   strncpy (ret, par_str + 1, par_len - 2);
   ret[par_len - 2] = '\0';
 
@@ -94,11 +87,7 @@ die_spaces_die (unsigned char * in_str)
   //Get the length of str, and allocate enough memory for ret_str.
   in_str_len = strlen ((const char *) in_str);
   ret_str = (unsigned char *) calloc (in_str_len + 1, sizeof(char));
-  if (!ret_str)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (ret_str, NULL);
 
   //Iterate through the string.
   int i, j = 0;
@@ -185,7 +174,8 @@ parse_parens (const unsigned char * in_str, const int init_pos,
       o_str = (unsigned char*) strchr ((const char *) in_str + paren_pos, '(');
       c_str = (unsigned char*) strchr ((const char *) in_str + paren_pos, ')');
 
-      //If the offset of c_str from in_str is less than the offset of o_str from in_str,
+      //If the offset of c_str from in_str,
+      // is less than the offset of o_str from in_str,
       //then this is a closing parentheses.
       if (c_str != NULL
 	  && (o_str == NULL || (c_str - in_str) < (o_str - in_str)))
@@ -198,22 +188,21 @@ parse_parens (const unsigned char * in_str, const int init_pos,
 	  o_paren++;
 	  paren_pos = o_str - in_str + 1;
 	}
-      else  //If both strings are NULL, then return -1.
-	return -1;
-
+      else
+	{
+	  //If both strings are NULL, then return -1.
+	  return -1;
+	}
     }
 
   paren_pos--;
 
   if (out_str)
     {
-      //Allocate enough room for out_str and copy the parentheses construct from in_str.
+      // Allocate enough room for out_str,
+      // and copy the parentheses construct from in_str.
       *out_str = (unsigned char*) calloc (paren_pos - init_pos + 2, sizeof (char));
-      if (!(*out_str))
-	{
-	  perror (NULL);
-	  return -2;
-	}
+      CHECK_ALLOC (*out_str, -2);
       strncpy (*out_str, in_str + init_pos, paren_pos - init_pos + 1);
       (*out_str)[paren_pos - init_pos + 1] = '\0';
     }
@@ -235,13 +224,12 @@ parse_parens (const unsigned char * in_str, const int init_pos,
 int
 reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned char ** out_str)
 {
-  //If the character at the given position is not a closing parentheses,
+  //If the character at the given position is not a closing parenthesis,
   //return -1 and set out_str to NULL.
-  if (in_str[init_pos] != 0x29)
-    {
-      out_str = NULL;
-      return -1;
-    }
+  if (out_str)
+    *out_str = NULL;
+  if (in_str[init_pos] != ')')
+    return -1;
 
   //The opening and closing parentheses count.
   int o_paren, c_paren;
@@ -260,11 +248,7 @@ reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned
   //Allocate enough memory, then copy the memory into tmp_str.
   in_len = strlen ((const char *) in_str);
   tmp_str = (unsigned char *) calloc (in_len, sizeof (char));
-  if (!tmp_str)
-    {
-      perror (NULL);
-      return -2;
-    }
+  CHECK_ALLOC (tmp_str, -2);
   strncpy (tmp_str, in_str, init_pos);
   tmp_str[init_pos] = '\0';
 
@@ -316,16 +300,12 @@ reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned
     {
 
       //Allocate space for out_str.
-      *out_str = (unsigned char *) calloc (init_pos - paren_pos + 1, sizeof (char));
-      if (!(*out_str))
-	{
-	  perror (NULL);
-	  return -2;
-	}
+      *out_str = (unsigned char *) calloc (init_pos - paren_pos + 2, sizeof (char));
+      CHECK_ALLOC (*out_str, -2);
 
       //When this is all finished, o_str will point to the string that is needed.
       strncpy (*out_str, in_str + paren_pos, init_pos - paren_pos + 1);
-      (*out_str)[init_pos - paren_pos + 1] = '\0';
+      (*out_str)[init_pos - paren_pos + 2] = '\0';
     }
 
   return paren_pos;
@@ -340,6 +320,9 @@ reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned
 int
 check_parens (const unsigned char *chk_str)
 {
+  if (chk_str[0] == '\0')
+    return 0;
+
   //Temporary strings for strstr.
   unsigned char * o_str, * c_str;
 
@@ -412,46 +395,17 @@ check_sides (const unsigned char * chk_str, const unsigned int init_pos)
   if (init_pos == 0)
     return 0;
 
-  unsigned int chk_len;
-  chk_len = strlen ((const char *) chk_str);
-
   unsigned char tmp_str[CL];
 
-  if (init_pos - CL >= 0)
-    {
-      strncpy (tmp_str, chk_str - CL, CL);
-
-      //Confirm that there is not another connective to the left.
-      if (!strncmp (tmp_str, AND, CL) || !strncmp (tmp_str, OR, CL)
-	  || !strncmp (tmp_str, CON, CL) || !strncmp (tmp_str, BIC, CL))
-	{
-	  return 0;
-	}
-
-      //Confirm that there is not a negation to the left.
-      if (!strncmp (tmp_str + (CL - NL), NOT, NL))
-	return 0;
-
-      //Confirm that there is not an opening parentheses to the left.
-      if (tmp_str[CL - 1] == '(')
-	return 0;
-    }
-
-  //This connective must not be the last character.
-  if (CL >= chk_len)
+  if (!isalnum (chk_str[init_pos - 1]) && chk_str[init_pos - 1] != ')'
+      && (init_pos < CL || strncmp (chk_str + init_pos - CL, NIL, CL)))
     return 0;
 
-  strncpy (tmp_str, chk_str + CL, CL);
+  //This connective must not be the last character.
+  if (chk_str[init_pos + CL] == '\0')
+    return 0;
 
-  //Confirm that there is not another connective to the right.
-  if (!strncmp (tmp_str, AND, CL) || !strncmp (tmp_str, OR, CL)
-      || !strncmp (tmp_str, CON, CL) || !strncmp (tmp_str, BIC, CL))
-    {
-      return 0;
-    }
-
-  //Confirm that there is not a closing parentheses to the right.
-  if (tmp_str[0] == ')')
+  if (!ISGOOD (chk_str + init_pos + CL) && !isalnum (chk_str[init_pos + CL]))
     return 0;
 
   return 1;
@@ -480,9 +434,7 @@ check_conns (const unsigned char * chk_str)
       if (!strncmp (chk_str + j, AND, CL) || !strncmp (chk_str + j, OR, CL)
 	  || !strncmp (chk_str + j, CON, CL) || !strncmp (chk_str + j, BIC, CL))
 	{
-	  chk_sides = check_sides (chk_str + j, j);
-	  if (chk_sides < 0)
-	    return -1;
+	  chk_sides = check_sides (chk_str, j);
 
 	  if (!chk_sides)
 	    return 0;
@@ -581,7 +533,14 @@ check_quants (const unsigned char * chk_str)
   return 1;
 }
 
-
+/* Gets a single generality.
+ *  input:
+ *    in_str - the string from which to get a generality.
+ *    in_pos - the initial position.
+ *    out_str - a string pointer that receives the generality.
+ *  output:
+ *    The end of the generality on success, -1 on memory error.
+ */
 int
 get_gen (unsigned char * in_str, int in_pos, unsigned char ** out_str)
 {
@@ -601,11 +560,7 @@ get_gen (unsigned char * in_str, int in_pos, unsigned char ** out_str)
     }
 
   *out_str = (unsigned char *) calloc (i - in_pos + 1, sizeof (char));
-  if (!(*out_str))
-    {
-      perror (NULL);
-      return -1;
-    }
+  CHECK_ALLOC (*out_str, -1);
 
   strncpy (*out_str, in_str + in_pos, i - in_pos);
   (*out_str)[i - in_pos] = '\0';
@@ -736,6 +691,9 @@ get_generalities (unsigned char * chk_str, unsigned char * conn, vec_t * vec)
 int
 check_text (unsigned char * text)
 {
+  if (text[0] == '\0')
+    return -2;
+
   unsigned char * eval_text;
   int ret;
 
@@ -785,6 +743,7 @@ check_text (unsigned char * text)
  *    -2 - Parenthesis Error
  *    -3 - Connective Error
  *    -4 - Quantifier Error
+ *    -5 - Construction Error
  */
 int
 check_generalities (unsigned char * text)
@@ -800,121 +759,153 @@ check_generalities (unsigned char * text)
 
   int text_len;
   int pos = 0;
+  int par_off = 0;
   int num_par = 0, cur_par = 0;
   text_len = strlen (text);
 
   unsigned char * tmp_str;
-  int tmp_pos;
+  int tmp_pos, ret_chk;
 
-  while (1)
+  if (text[0] == '\0')
+    return -5;
+
+  if (!ISGOOD (text)
+      && !(strstr (text, AND) || strstr (text, OR)
+	   || strstr (text, CON) || strstr (text, BIC)))
     {
-      if (text_len >= NL && !strncmp (text + pos, NOT, NL))
+      if (strstr (text, ELM) || strchr (text, '=') || strchr (text, '<'))
 	{
-	  while (pos < text_len && !strncmp (text + pos, NOT, NL))
-	    pos += NL;
-
-	  if (pos == text_len)
-	    {
-	      return -3;
-	    }
-	}
-
-      if (text_len >= CL
-	  && (!strncmp (text + pos, UNV, CL) || !strncmp (text + pos, EXL, CL)))
-	{
-	  pos += CL + 1;
-	}
-
-      if (text[pos] == '(')
-	{
-	  tmp_pos = parse_parens (text, pos, &tmp_str);
-	  if (tmp_pos == -2)
+	  ret_chk = check_infix (text, 1);
+	  if (ret_chk == -1)
 	    return -1;
 
-	  num_par++;
-	  if (tmp_pos == text_len - num_par)
-	    {
-	      // The scope of the paren construct is the entire sentence,
-	      // so update the counters, and restart the loop.
-	      cur_par = pos;
-	      pos++;
-	      free (tmp_str);
-	      continue;
-	    }
-	  else
-	    {
-	      // Doesn't span the entire sentence, so generalities must be obtained.
-	      break;
-	    }
+	  if (ret_chk == 0)
+	    return 0;
 	}
 
-      break;
+      return -5;
+    }
+
+  tmp_pos = get_gen (text, 0, &tmp_str);
+  if (tmp_pos == -1)
+    return -1;
+
+  if (!strcmp (tmp_str, text))
+    {
+      // This is the entire scope, so strip away the preceding symbol,
+      //  if one exists.
+
+      if (!strncmp (text, NOT, NL))
+	{
+	  if (text[NL] == '\0')
+	    return -3;
+
+	  return check_generalities (text + NL);
+	}
+
+      if (!strncmp (text, UNV, CL) || !strncmp (text, EXL, CL))
+	{
+	  pos = CL;
+	  while ((islower(text[pos]) || isdigit(text[pos]))
+		 && text[pos] != '\0')
+	    pos++;
+	  if (text[pos] == '\0')
+	    return -4;
+
+	  if (strncmp (text + pos, UNV, CL) && strncmp (text + pos, EXL, CL)
+	      && strncmp (text + pos, NOT, NL) && text[pos] != '(')
+	    return -4;
+
+	  return check_generalities (text + pos);
+	}
+
+      if (text[0] == '(')
+	{
+	  tmp_str = elim_par (text);
+	  if (!tmp_str)
+	    return -1;
+
+	  ret_chk = check_generalities (tmp_str);
+	  if (ret_chk == -1)
+	    return -1;
+	  free (tmp_str);
+	  return ret_chk;
+	}
+
+      if (text[0] != '(')
+	{
+	  ret_chk = check_symbols (text, 1);
+	  if (ret_chk == -1)
+	    return -1;
+
+	  if (ret_chk == -2)
+	    return -5;
+
+	  return 0;
+	}
     }
 
   // Get generalities across the parenthesis construct.
 
   int gg;
-  unsigned char * conn, * elims;
+  unsigned char conn[CL + 1], * elims;
   vec_t * gens;
 
-  if (text[cur_par] == '(')
-    {
-      tmp_pos = parse_parens (text, cur_par, &tmp_str);
-      if (tmp_pos == -2)
-	return -1;
-
-      elims = elim_par (tmp_str);
-      if (!elims)
-	return -1;
-      free (tmp_str);
-    }
-  else
-    {
-      elims = text;
-    }
-
-  if (strlen (elims) == 0)
-    {
-      return -2;
-    }
-
-  conn = (unsigned char *) calloc (CL + 1, sizeof (char *));
   gens = init_vec (sizeof (char *));
+  if (!gens)
+    return -1;
+
   conn[0] = '\0';
-  gg = get_generalities (elims, conn, gens);
+  gg = get_generalities (text, conn, gens);
   if (gg == -3)
     return -1;
 
+  if (gens->num_stuff == 1 || gens->num_stuff == 0)
+    {
+      destroy_str_vec (gens);
+
+      int ret_chk = check_symbols (text, 1);
+      if (ret_chk == -1)
+	return -1;
+
+      if (ret_chk == -2)
+	return -5;
+
+      return 0;
+    }
+
   if (gg == -2)
     {
       // Connective Error.
+      REPORT ();
       return -3;
     }
 
   if (gens->num_stuff >= 3)
     {
-      if (!strncmp (conn, CON, CL) || !strncmp (conn, BIC, CL))
+      if (!strcmp (conn, CON) || !strcmp (conn, BIC))
 	{
 	  // Connective Error.
+	  REPORT ();
+	  destroy_str_vec (gens);
 	  return -3;
 	}
     }
 
-  if (gens->num_stuff == 1)
-    return 0;
-
-  int i, ret_chk;
+  int i;
   unsigned char * cur_gen;
   for (i = 0; i < gens->num_stuff; i++)
     {
       cur_gen = vec_str_nth (gens, i);
       ret_chk = check_generalities (cur_gen);
+      if (ret_chk == -1)
+	return -1;
+
       if (ret_chk != 0)
 	{
 	  destroy_str_vec (gens);
 	  return ret_chk;
 	}
-
     }
 
   destroy_str_vec (gens);
@@ -922,10 +913,251 @@ check_generalities (unsigned char * text)
   return 0;
 }
 
+/* Checks a predicate / function symbol.
+ *  input:
+ *    in_str - the string to check.
+ *    pred - 1 if a predicate, 0 if a function symbol.
+ *  output:
+ *    0 - success.
+ *    -1 - Memory error
+ *    -2 - Error
+ */
+int
+check_symbols (unsigned char * in_str, int pred)
+{
+  // Check for infix predicates / function symbols.
+
+  int ret_chk;
+
+  if (!strcmp (in_str, NIL))
+    return 0;
+
+  ret_chk = check_infix (in_str, pred);
+  if (ret_chk != -3)
+    return ret_chk;
+
+  if (pred)
+    {
+      if (!strcmp (in_str, TAU) || !strcmp (in_str, CTR))
+	return 0;
+
+      if (!isupper (in_str[0]))
+	return -2;
+    }
+  else
+    {
+      if (!islower (in_str[0]) && !isdigit (in_str[0]))
+	return -2;
+    }
+
+  int pos = 1;
+
+  while (in_str[pos] != '(' && in_str[pos] != '\0')
+    {
+      if (!ISLEGIT (in_str[pos]))
+	return -2;
+
+      pos++;
+    }
+
+  if (in_str[pos] == '\0')
+    return 0;
+
+  pos++;
+
+  if (in_str[pos] == ')')
+    return -2;
+
+  int cur_pos;
+
+  while (in_str[pos] != ')')
+    {
+      if (!strncmp (in_str + pos, NIL, CL))
+	{
+	  cur_pos = pos + CL;
+	}
+      else
+	{
+	  cur_pos = pos;
+
+	  while (ISLEGIT (in_str[cur_pos]))
+	    cur_pos++;
+	}
+
+      if (!ISSEP (in_str[cur_pos]))
+	return -2;
+
+      int got_infix = 0, done = 0, cont = 0;
+
+      while (1)
+	{
+	  int tmp_pos;
+	  unsigned char * tmp_str;
+
+	  switch (in_str[cur_pos])
+	    {
+	    case '(':
+	      tmp_pos = parse_parens (in_str, cur_pos, NULL);
+	      cur_pos = tmp_pos + 1;
+	      break;
+
+	    case '+':
+	    case '*':
+	      cur_pos++;
+	      break;
+
+	    case ')':
+	    case ',':
+	      tmp_str = (unsigned char *) calloc (cur_pos - pos + 1, sizeof (char));
+	      CHECK_ALLOC (tmp_str, -1);
+
+	      strncpy (tmp_str, in_str + pos, cur_pos - pos);
+	      tmp_str[cur_pos - pos] = '\0';
+
+	      ret_chk = check_symbols (tmp_str, 0);
+	      if (ret_chk == -1)
+		return -1;
+
+	      free (tmp_str);
+	      if (ret_chk == -2)
+		return -2;
+	      done = 1;
+	      if (in_str[cur_pos] == ',')
+		cur_pos++;
+	      break;
+	    }
+
+	  if (ISSEP (in_str[cur_pos]) && in_str[cur_pos] != ')')
+	    {
+	      if (in_str[cur_pos - 1] != ')')
+		return -2;
+
+	      if (in_str[cur_pos] == '(')
+		return -2;
+	    }
+
+	  if (done)
+	    break;
+	}
+
+      pos = cur_pos;
+    }
+
+  if (in_str[pos + 1] != '\0')
+    return -2;
+
+  return 0;
+}
+
+
+/* Checks a predicate / function infix symbol.
+ *  input:
+ *    in_str - the string to check.
+ *    pred - 1 if a predicate, 0 if a function symbol.
+ *  output:
+ *    0 - success.
+ *    -1 - Memory error
+ *    -2 - Error
+ *    -3 - No infix symbols found
+ */
+int
+check_infix (unsigned char * in_str, int pred)
+{
+  // Check for an infix predicate.
+
+  unsigned char * lsen, * rsen;
+
+  if (pred)
+    {
+      // Determine which predicate symbol is being used, if any.
+
+      unsigned char * sym;
+
+      char * ss_elm, * ss_id, * ss_lt;
+
+      ss_elm = strstr (in_str, ELM);
+      ss_id = strchr (in_str, '=');
+      ss_lt = strchr (in_str, '<');
+
+      if (!ss_elm && !ss_id && !ss_lt)
+	return -3;
+
+      if (ss_elm)
+	{
+	  if (ss_id || ss_lt)
+	    return -2;
+
+	  sym = ELM;
+	}
+
+      if (ss_id)
+	{
+	  if (ss_lt)
+	    return -2;
+
+	  sym = "=";
+	}
+
+      if (ss_lt)
+	sym = "<";
+
+      unsigned char * sym_str;
+      int in_len;
+
+      in_len = strlen (in_str);
+      sym_str = (unsigned char *) strstr (in_str, sym);
+
+      lsen = (unsigned char *) calloc (sym_str - in_str + 1, sizeof (char));
+      CHECK_ALLOC (lsen, -1);
+
+      strncpy (lsen, in_str, sym_str - in_str);
+      lsen[sym_str - in_str] = '\0';
+
+      rsen = sym_str + strlen (sym);
+      if (strstr (rsen, sym))
+	return -2;
+
+      int ret_chk;
+
+      ret_chk = check_symbols (lsen, 0);
+      if (ret_chk == -1)
+	return -1;
+
+      free (lsen);
+      if (ret_chk == -2)
+	return -2;
+
+      ret_chk = check_symbols (rsen, 0);
+      if (ret_chk == -1)
+	return -1;
+
+      if (ret_chk == -2)
+	return -2;
+    }
+  else
+    {
+      char * ss_add, * ss_mul;
+
+      ss_add = strchr (in_str, '+');
+      ss_mul = strchr (in_str, '*');
+
+      if (!ss_add && !ss_mul)
+	return -3;
+    }
+
+  return 0;
+}
+
+/* Determines the sexpr connective of a standard connective.
+ *  input:
+ *    conn - the standard connective.
+ *  output:
+ *    The sexpr connective that corresponds to the connective.
+ */
 unsigned char *
 conn_to_sexpr (unsigned char * conn)
 {
-  unsigned char * ret_conn;
+  unsigned char * ret_conn = NULL;
 
   if (!strcmp (conn, AND))
     ret_conn = sexpr_conns.and;
@@ -957,6 +1189,12 @@ conn_to_sexpr (unsigned char * conn)
   return ret_conn;
 }
 
+/* Converts a string to a sexpr string.
+ *  input:
+ *    in_str - the string to convert.
+ *  output:
+ *    The sexpr form of the input string.
+ */
 unsigned char *
 convert_sexpr (unsigned char * in_str)
 {
@@ -1003,12 +1241,7 @@ convert_sexpr (unsigned char * in_str)
 
 	  out_str = (unsigned char *) calloc (strlen (sexpr_str) + sexpr_conns.nl + 4,
 					      sizeof (char));
-	  if (!out_str)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
-	  
+	  CHECK_ALLOC (out_str, NULL);
 	  out_pos += sprintf (out_str + out_pos, "(%s %s)",
 			      sexpr_conns.not, sexpr_str);
 	  free (sexpr_str);
@@ -1030,12 +1263,7 @@ convert_sexpr (unsigned char * in_str)
 	  free (par_in_str);
 
 	  out_str = (unsigned char *) calloc (strlen (sexpr_str) + 1, sizeof (char));
-	  if (!out_str)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
-
+	  CHECK_ALLOC (out_str, NULL);
 	  strcpy (out_str, sexpr_str);
 	  free (sexpr_str);
 	  return out_str;
@@ -1059,11 +1287,7 @@ convert_sexpr (unsigned char * in_str)
 	    }
 
 	  quant_var = (unsigned char *) calloc (i - CL + 1, sizeof (char));
-	  if (!quant_var)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
+	  CHECK_ALLOC (quant_var, NULL);
 
 	  strncpy (quant_var, in_str + CL, i - CL);
 	  quant_var[i - CL] = '\0';
@@ -1071,11 +1295,7 @@ convert_sexpr (unsigned char * in_str)
 	  unsigned char * quant_in_str, * sexpr_str;
 
 	  quant_in_str = (unsigned char *) calloc (in_len - i + 1, sizeof (char));
-	  if (!quant_in_str)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
+	  CHECK_ALLOC (quant_in_str, NULL);
 
 	  strcpy (quant_in_str, in_str + i);
 	  sexpr_str = convert_sexpr (quant_in_str);
@@ -1087,12 +1307,7 @@ convert_sexpr (unsigned char * in_str)
 
 	  alloc_size = strlen (quant_var) + sexpr_conns.cl + strlen (sexpr_str) + 7;
 	  out_str = (unsigned char *) calloc (alloc_size, sizeof (char));
-	  if (!out_str)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
-
+	  CHECK_ALLOC (out_str, NULL);
 	  out_pos += sprintf (out_str + out_pos, "((%s %s) %s)",
 			      sexpr_quant, quant_var, sexpr_str);
 	  free (sexpr_str);
@@ -1122,26 +1337,23 @@ convert_sexpr (unsigned char * in_str)
 	  if (!strcmp (tmp_str, TAU) || !strcmp (tmp_str, CTR))
 	    {
 	      out_str = (unsigned char *) calloc (sexpr_conns.cl + 1, sizeof (char));
-	      if (!out_str)
-		{
-		  perror (NULL);
-		  return NULL;
-		}
+	      CHECK_ALLOC (out_str, NULL);
 
 	      if (!strcmp (tmp_str, TAU))
 		strcpy (out_str, sexpr_conns.tau);
 	      else
 		strcpy (out_str, sexpr_conns.ctr);
 	    }
+	  else if (!strcmp (tmp_str, NIL))
+	    {
+	      out_str = (unsigned char *) calloc (sexpr_conns.cl + 1, sizeof (char));
+	      CHECK_ALLOC (out_str, NULL);
+	      strcpy (out_str, sexpr_conns.nil);
+	    }
 	  else
 	    {
 	      out_str = (unsigned char *) calloc (tmp_len + 1, sizeof (char));
-	      if (!out_str)
-		{
-		  perror (NULL);
-		  return NULL;
-		}
-
+	      CHECK_ALLOC (out_str, NULL);
 	      strcpy (out_str, tmp_str);
 	    }
 	}
@@ -1184,12 +1396,7 @@ convert_sexpr (unsigned char * in_str)
 	  destroy_str_vec (arg_vec);
 
 	  out_str = (unsigned char *) calloc (alloc_size + 2, sizeof (char));
-	  if (!out_str)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
-
+	  CHECK_ALLOC (out_str, NULL);
 	  out_pos += sprintf (out_str + out_pos, "(%s", pred_in_str);
 	  free (pred_in_str);
 
@@ -1240,11 +1447,7 @@ convert_sexpr (unsigned char * in_str)
 
   destroy_str_vec (gg_vec);
   out_str = (unsigned char *) calloc (alloc_size + 2, sizeof (char));
-  if (!out_str)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (out_str, NULL);
 
   out_pos += sprintf (out_str, "(%s", sexpr_conn);
 
@@ -1258,26 +1461,48 @@ convert_sexpr (unsigned char * in_str)
   out_pos += sprintf (out_str + out_pos, ")");
 
   destroy_str_vec (sexpr_gens);
-
   return out_str;
 }
 
+/* Converts a string from infix to prefix.
+ *  input:
+ *    in_str - the string to convert.
+ *  output:
+ *    The string in prefix form.
+ */
 unsigned char *
 infix_to_prefix (unsigned char * in_str)
 {
   unsigned char * out_str;
 
-  int i, pos, in_len;
+  int i, pos, in_len, got_nil = 0;
   unsigned char * lsen, * rsen, * sym;
 
   in_len = strlen (in_str);
   i = 0;
   pos = 0;
 
+  if (!strcmp (in_str, NIL))
+    {
+      out_str = (unsigned char *) calloc (in_len + 1, sizeof (char));
+      CHECK_ALLOC (out_str, NULL);
+
+      strcpy (out_str, in_str);
+      return out_str;
+    }
+
+
   while (i < in_len)
     {
       int ret_check;
 
+      if (!strncmp (in_str + i, NIL, CL))
+	{
+	  i += 3;
+	  got_nil = 1;
+	  continue;
+	}
+
       while (islower (in_str[i]) || isdigit (in_str[i]))
 	i++;
 
@@ -1298,12 +1523,7 @@ infix_to_prefix (unsigned char * in_str)
       if (in_str[i] == '=' || in_str[i] == '<' || !strncmp (in_str + i, ELM, CL))
 	{
 	  lsen = (unsigned char *) calloc (i + 1, sizeof (char));
-	  if (!lsen)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
-
+	  CHECK_ALLOC (lsen, NULL);
 	  strncpy (lsen, in_str + pos, i);
 	  lsen[i] = '\0';
 
@@ -1325,11 +1545,7 @@ infix_to_prefix (unsigned char * in_str)
     }
 
   rsen = (unsigned char *) calloc (in_len - pos + 1, sizeof (char));
-  if (!rsen)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (rsen, NULL);
   strcpy (rsen, in_str + pos);
 
   unsigned char * c_lsen, * c_rsen;
@@ -1347,18 +1563,14 @@ infix_to_prefix (unsigned char * in_str)
 
   out_str = (unsigned char *) calloc (strlen (c_lsen) + strlen (c_rsen) + 8,
 				      sizeof (char));
-  if (!out_str)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (out_str, NULL);
 
   int out_pos;
 
   if (!strncmp (in_str + i, ELM, CL))
     {
-      strncpy (out_str, ELM, CL);
-      out_pos = CL;
+      strncpy (out_str, sexpr_conns.nil, sexpr_conns.cl);
+      out_pos = sexpr_conns.cl;
     }
   else
     {
@@ -1372,6 +1584,12 @@ infix_to_prefix (unsigned char * in_str)
   return out_str;
 }
 
+/* Converts a function from infix to prefix.
+ *  input:
+ *    in_str - the string to convert.
+ *  output:
+ *    The converted string.
+ */
 unsigned char *
 infix_to_prefix_func (unsigned char * in_str)
 {
@@ -1384,6 +1602,15 @@ infix_to_prefix_func (unsigned char * in_str)
   i = 0;
   pos = 0;
 
+  if (!strcmp (in_str, NIL))
+    {
+      out_str = (unsigned char *) calloc (in_len + 1, sizeof (char));
+      CHECK_ALLOC (out_str, NULL);
+
+      strcpy (out_str, in_str);
+      return out_str;
+    }
+
   while (i < in_len)
     {
       int ret_check;
@@ -1408,11 +1635,7 @@ infix_to_prefix_func (unsigned char * in_str)
       if (i != 0 && (in_str[i] == '+' || in_str[i] == '*'))
 	{
 	  lsen = (unsigned char *) calloc (i + 1, sizeof (char));
-	  if (!lsen)
-	    {
-	      perror (NULL);
-	      return NULL;
-	    }
+	  CHECK_ALLOC (lsen, NULL);
 
 	  strncpy (lsen, in_str + pos, i);
 	  lsen[i] = '\0';
@@ -1428,22 +1651,14 @@ infix_to_prefix_func (unsigned char * in_str)
   if (i >= in_len)
     {
       out_str = (unsigned char *) calloc (in_len + 1, sizeof (char));
-      if (!out_str)
-	{
-	  perror (NULL);
-	  return NULL;
-	}
+      CHECK_ALLOC (out_str, NULL);
 
       strcpy (out_str, in_str);
       return out_str;
     }
 
   rsen = (unsigned char *) calloc (in_len - pos + 1, sizeof (char));
-  if (!rsen)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (rsen, NULL);
   strcpy (rsen, in_str + pos);
 
   unsigned char * c_lsen, * c_rsen;
@@ -1461,11 +1676,7 @@ infix_to_prefix_func (unsigned char * in_str)
 
   out_str = (unsigned char *) calloc (strlen (c_lsen) + strlen (c_rsen) + 5,
 				      sizeof (char));
-  if (!out_str)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (out_str, NULL);
 
   sprintf (out_str, "%c(%s,%s)", in_str[i], c_lsen, c_rsen);
   free (c_lsen);
@@ -1473,23 +1684,27 @@ infix_to_prefix_func (unsigned char * in_str)
   return out_str;
 }
 
+/* Gets the arguments from a predicate or function.
+ *  input:
+ *    in_str - the string to check.
+ *    in_pos - the initial position.
+ *    sym - a pointer to a string that receives the symbol.
+ *    args - a string vector that receives the arguments.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
 int
 get_pred_func_args (unsigned char * in_str, int in_pos,
 		    unsigned char ** sym, vec_t * args)
 {
   int i;
 
-  i = 0;
+  i = in_pos;
   while (in_str[i] != '(')
     i++;
 
   *sym = (unsigned char *) calloc (i + 1, sizeof (char));
-  if (!(*sym))
-    {
-      perror (NULL);
-      return -1;
-    }
-
+  CHECK_ALLOC (*sym, -1);
   strncpy (*sym, in_str, i);
   (*sym)[i] = '\0';
 
@@ -1497,7 +1712,7 @@ get_pred_func_args (unsigned char * in_str, int in_pos,
   unsigned char * tmp_str;
   int tmp_pos;
 
-  tmp_pos = parse_parens (in_str, in_pos + i, &tmp_str);
+  tmp_pos = parse_parens (in_str, i, &tmp_str);
   if (tmp_pos == -2)
     return -1;
 
@@ -1532,11 +1747,7 @@ get_pred_func_args (unsigned char * in_str, int in_pos,
 	  unsigned char * new_arg;
 
 	  new_arg = (unsigned char *) calloc (i - pos + 1, sizeof (char));
-	  if (!new_arg)
-	    {
-	      perror (NULL);
-	      return -1;
-	    }
+	  CHECK_ALLOC (new_arg, -1);
 
 	  strncpy (new_arg, elim_str + pos, i - pos);
 	  new_arg[i - pos] = '\0';
@@ -1559,3 +1770,33 @@ get_pred_func_args (unsigned char * in_str, int in_pos,
   return 0;
 }
 
+/* Creates a constant definition.
+ *  input:
+ *    in_str - a string that contains one number.
+ *  output:
+ *    A string containing a definition of the number in seqlog.
+ */
+unsigned char *
+new_const (char * in_str)
+{
+  int n, i;
+  unsigned char * out_str;
+  int out_pos, alloc_size;
+
+  sscanf (in_str, "%i", &n);
+
+  alloc_size = CL + 14 + (3 * n) + (log (n) + 1);
+  out_str = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+  CHECK_ALLOC (out_str, NULL);
+
+  out_pos += sprintf (out_str + out_pos, "%s(v(n,", UNV);
+  for (i = 0; i < n; i++)
+    out_pos += sprintf (out_str + out_pos, "s(");
+
+  out_pos += sprintf (out_str + out_pos, "z(x)");
+  for (i = 0; i < n; i++)
+    out_pos += sprintf (out_str + out_pos, ")");
+  out_pos += sprintf (out_str + out_pos, ") = %i)", n);
+
+  return out_str;
+}
